﻿using System;
using System.Reflection;
using TenCrowns.AppCore;
using TenCrowns.GameCore;
using HarmonyLib;

namespace UnlimitedAutosaves
{
    public class UnlimitedAutosavesClass : ModEntryPointAdapter
    {
        public const string MY_HARMONY_ID = "Fim.UnlimitedAutosaves.patch";
        public Harmony harmony;

        public override void Initialize(ModSettings modSettings)
        {
            harmony = new Harmony(MY_HARMONY_ID);
            harmony.PatchAll();
        }
        public override void Shutdown()
        {
            harmony.UnpatchAll(MY_HARMONY_ID);
        }
    }

    //Game.cs, Class Game, original code:
    //public virtual int getNumAutosaves()
    //{
    //    return Math.Max(1, mzGameParameters.numAutosaves);
    //}
    [HarmonyPatch(typeof(Game), nameof(Game.getNumAutosaves))]
    public class NumAutoSavesPatch
    {
        static void Postfix(ref Game __instance, ref int __result)
        {
            __result = Math.Max(__result, __instance.infos().Globals.NUM_AUTOSAVES); //There's a GlobalsInt for it but it gets ignored -_-
        }
    }

    //Game.cs, Class Game, original code:
    //public virtual string getAutoSavePath(int iTurn)
    //{
    //    return getSaveFolder() + "/" + String.Format(Constants.AUTO_SAVE_FORMAT, getSaveID(getTurnSaveID(iTurn).ToStringCached()));
    //}
    [HarmonyPatch(typeof(Game), nameof(Game.getAutoSavePath), new Type[] { typeof(int) })]
    public class AutoSavePathPatch
    {
        static bool Prefix(ref Game __instance, ref string __result, int iTurn)
        {
            //Using System.Reflection directly would look like this:
            //System.Reflection.MethodInfo mInfo = __instance.GetType().GetMethod("getTurnSaveID", BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Static, null, new Type[] { typeof(int) }, null);
            MethodInfo getTurnSaveID = AccessTools.Method(typeof(Game), "getTurnSaveID", new Type[] { typeof(int) });
            object[] argsGetTurnSaveID = new object[] { iTurn };
            int iTurnSaveID = (int)getTurnSaveID.Invoke(__instance, argsGetTurnSaveID);

            MethodInfo getSaveID = AccessTools.Method(typeof(Game), "getSaveID", new Type[] { typeof(string) });
            object[] argsGetSaveID = new object[] { iTurnSaveID.ToString("000") };
            string saveID = (String)getSaveID.Invoke(__instance, argsGetSaveID);

            MethodInfo getSaveFolder = AccessTools.Method(typeof(Game), "getSaveFolder");
            __result = (String)getSaveFolder.Invoke(__instance, null) + "/" + String.Format(Constants.AUTO_SAVE_FORMAT, saveID);

            //don't run the original method
            return false;
        }
    }
}